unit uPropCalc;

//
//
//  MicroMUF:
//
//  This is a Delphi conversion of the public-domain MicroMUF
//  BASIC program, originally developed by the BBC and further
//  refined by R. Nederland.
//
//  MicroMUF.bas is included in the comments at the end of this
//  module.
//
//                             Tom Lackamp  AB9B
//                             February, 2010
//
//
//  To integrate this module into your own program, feed the input
//  values into the StartCalc routine, and process the calculator's
//  output in the ShowResults routine.
//
//  Input values are source and destination latitude and longitude,
//  solar flux value, and month
//
//  Output values are Maximum Usable Frequency (MUF) and Lowest Usable  
//  Frequency (LUF), in MHz, stored in arrays[0..24] indexed by UTC hours.
//
//  Other output values are distance in km, great circle bearings to and from
//  the destination, and the number of hops in the path.
//
//  Input:
//
//    YR : Source longitude, -180.0..+180.0, where positive = west longitude
//    XR : Source latitude, -90.0..+90.0, where positive = north longitude
//
//    YT : Destination longitude, as above
//    XT : Destination latitude, as above
//
//     F : Solar flux, positive double
//
//     M : Month, 1..12
//
//  Output:
//
//    MUF[0..24] : array of double,  frequency (MHz) vs UTC hours, 0..24
//    LUF[0..24] : array of double,  as above    
//
//    L, distance (km)
//
//    U, bearing from destination toward source, great circle true
//  
//    MyAz, bearing from source toward destination
//
//    N, hops
//

interface

uses
  SysUtils,Math;

  procedure StartCalc;

implementation

uses
  frmMain;

var

  MUF : array[0..24] of Double;
  LUF : array[0..24] of Double;
  MyAz : Double;
  A, N, YT, XT, YR, XR, P, H, XS, MH, TY,
  R, M, X, Y, LA, LS, HP, U, XN, YI, YF, FO,
  SF, FE, SE, RE, CP, RD, D, YN, Z, TL,
  K, HO, VE, L, XZ, MF, FF, YZ, YG, ZO,
  XH, SX, WX, LH, LM, AB, Q : Double;

  T : Integer;

  Ax : String;


procedure Init;
//
//  Initialize BASIC variables
//
begin

  MyAz := 0;
  YZ := 0; YG := 0; ZO := 0; Z := 0; TL := 0;  MH := 0; YF := 0;
  H := 0; XS := 0; XN := 0; YI := 0; YN := 0; TY := 0; FO := 0;
  A := 0; N := 0; YT := 0; XT := 0; YR := 0; XR := 0; P := 0;
  R := 0; M := 0; X := 0; Y := 0; LA := 0; LS := 0; HP := 0;
  SF := 0; FE := 0; SE := 0; RE := 0; CP := 0; Q := 0; U := 0;
  K := 0; HO := 0; VE := 0; L := 0; XZ := 0; MF := 0; FF := 0;
  XH := 0; SX := 0; WX := 0; LH := 0; LM := 0; AB := 0; T := 0;
  Ax := '';
  RD := Pi/180;
  D := 180/Pi;

end;

function FluxToSpots( Flux : Double ) : Double;
//
//   Converts solar flux to sunspot number
//   Source:  IPS Radio & Space Services
//
var
  FD : Double;

begin

  FD := Flux - 67;
  Result := 1.61 * FD - Power( (0.0733*FD), 2 ) +
            Power( (0.0240*FD), 3 );

end;

procedure CalcMyAz;
//
//  Calculate true great circle azimuth from
//  source to destination
//
var
  Qx, Xx, LAx, Lx, Ux : Double;

begin

  Qx := SIN(XR*RD)*SIN(XT*RD);
  Xx := Qx + COS(XR*RD) * COS(XT*RD) * COS(YR*RD-YT*RD);
  LAx := -1 * ArcTan(Xx/(SQRT(-1*Xx*Xx+1+1E-12))) + (Pi/2);
  LAx := LAx*D;
  Lx := 111.1*LAx;
  Qx := (SIN(XT*RD)- SIN(XR*RD)*COS(LAx*RD));
  Xx := Qx/COS(XR*RD)/SIN(LAx*RD);
  Ux := -1 * ArcTan(Xx/(SQRT(-1 * Xx*Xx +1+1E-12)))+(Pi/2);
  Ux := Ux*D;

  if (YR-YT) <= 0 then
    Ux := 360-Ux;
  if ABS(YR-YT) >= 180 then
    Ux := 360-Ux;

  MyAz := Ux;

end;

procedure GeoCalc;
//
//  Geometry calculations
//
begin

  Q := SIN(XT*RD)*SIN(XR*RD);
  X := Q + COS(XT*RD) * COS(XR*RD) * COS(YT*RD-YR*RD);
  LA := -1 * ArcTan(X/(SQRT(-1*X*X+1+1E-12))) + (Pi/2);
  LA := LA*D;
  L := 111.1*LA;
  Q := (SIN(XR*RD)- SIN(XT*RD)*COS(LA*RD));
  X := Q/COS(XT*RD)/SIN(LA*RD);
  U := -1 * ArcTan(X/(SQRT(-1 * X*X +1+1E-12)))+(Pi/2);
  U := U*D;
  if (YT-YR) <= 0 then
    U := 360-U;
  if ABS(YT-YR) >= 180 then
    U := 360-U;
  H := 275+R/2;
  XS := 23.4*COS(30*(M-6.25)*RD);
  N := N+1;
  LH := L/N;
  while LH>4500 do
    begin
      N := N+1;
      LH := L/N;
    end;
  LM := LA/N;
  A := ArcTan((COS(0.5*LM*RD)-6367/(H+6367))/( SIN(0.5*LM*RD)));
  A := A*D;
  while A<1.5 do
    begin
      N := N+1;
      LH := L/N;
      while LH>4500 do
        begin
          N := N+1;
          LH := L/N;
        end;
      LM := LA/N;
      A := ArcTan((COS(0.5*LM*RD)-6367/(H+6367))/(SIN(0.5*LM*RD)));
      A := A*D;
    end;

end;

procedure InterimLatLong;
begin

  Q := COS(U*RD)*COS(XT*RD)*SIN(K*LM*RD);
  X := Q+SIN(XT*RD)*COS(K*LM*RD);
  XN := ArcTan(X/(SQRT(-1 * X*X+1+1E-12)));
  XN := XN*D;
  Q := (COS(K*LM*RD)-SIN(XT*RD)*SIN(XN*RD));
  X := Q/(COS(XT*RD)*COS(XN*RD));
  YI := -1 * ArcTan( X/(SQRT(-1 * X*X + 1 + 1E-12)) ) + (Pi/2);
  YI := YI*D;
  if U<180 then
    YI := -YI;
  YN := YT+YI;
  if YN>180 then
    YN := YN-360;
  if YN<-180 then
    YN := YN+360;

end;

procedure MiniF2;
begin

  YZ := YN;
  if YN<-160 then
    YZ := YN+360;
  YG := (20-YZ)/50;
  ZO := 20*YG/Power(1+YG+YG,2)+5*Power((1-YG/7),2);
  Z := XN-ZO;
  TL := T-YN/15;
  if TL>24 then
    TL := TL-24;
  if TL<0 then
    TL := TL+24;
  MH := M;
  if Z <= 0 then
  begin
    Z := -1 * Z;
    MH := M+6;
  end;
  XH := COS(30*(MH-6.5)*RD);  // 1 WEEK DELAY ON EQUINOXES
  SX := (ABS(XH)+XH)/2; // F LAYER LOCAL SUMMER VAR.
  WX := (ABS(XH)-XH)/2;  // F LAYER LOCAL WINTER VAR.
  if Z>77.5 then
    Z := 77.5;
  TY := TL;
  if TY<5 then
    TY := TL+24;
  YF := (TY-14-SX*2+WX*2-R/175)*(7-SX*3+WX*4-R/(150-WX*75));
  if ABS(YF)>60 then
    YF := 60;
  X := (1+R/(175+SX*175));
  FO := 6.5*X*COS(YF*RD)*Power(COS((Z-SX*5+WX*5)*RD),0.5);
  SF := Power(1-Power(COS(A*RD)*6367/(6367+H),2),-0.5);
  FF := FO*SF; // MUF AT CONTROL POINT

end;

procedure ELayer;
//
//  E-Layer and Absorption calculations
//
var
  Dummy : Integer;
  Label L2560, L2570;

begin

  Q := SIN(XN*RD)*SIN(XS*RD);
  X := Q+COS(XN*RD)*COS(XS*RD)*COS((YN-15*(T-12))* RD);
  XZ := (-1 * ArcTan(X/(SQRT(-X*X+1+1E-12)))+Pi/2)* D;
  if XZ>85 then
    goto L2560;
  FE := 3.4*(1+0.0016*R)*Power(COS(XZ*RD),0.3);
  goto L2570;
L2560:
  FE := 3.4*(1+0.0016*R)*Power((XZ-80),-0.5);
L2570:
  SE := Power(1-0.965*Power(COS(A*RD),2),-0.5);
  LS := 0.028*Power(FE,2)*SE;

end;

procedure CalcOut;
//
//  Calculate MUF and LUF values, store in MUF and LUF arrays
//
begin

  for T := 1 to 24 do
    begin
      AB := 0;
      K := 0.5;
      InterimLatLong;
      MiniF2;
      MF := FF;
      K := N - 0.5;
      InterimLatLong;
      MiniF2;
      if FF<MF then
        MF := FF;
      MUF[T-1] := MF;
      K := 0.25;
      while K < (N-0.25) do
        begin
          InterimLatLong;
          ELayer;
          AB := AB+LS;
          K := K + 0.5;
        end;
      AB := AB + 2;
      LUF[T-1] := AB;
    end;

  MUF[24] := MUF[0];     // ...so the graph wraps around
  LUF[24] := LUF[0];

end;

procedure ShowResults;
//
//                                            ***  PROCESS YOUR OUTPUT HERE ***
//
//  Process the calculator's output
//
var
  i, Hops : Integer;

begin

  //  Graph the output MUF & LUF vs UTC

{

  for i := 0 to 24 do begin

    // Plot MUF[i]

    // Plot LUF[i]

  end;

}

  Main.Plot(MUF,LUF);

  //  Distance

    //  your_output_display_object := L, where L = distance in km

  Main.edDistance.Text := FloatToStrF(L,ffNumber,7,0);

  //  Bearing from destination toward source, great circle true

    //  your_output_display_object := U

  Main.edAzFromDest.Text := FloatToStrF(U,ffNumber,7,0);

  //  Bearing from source toward destination, great circle true

    // your_output_dispay_object := MyAz

  Main.edAzFromSource.Text := FloatToStrF(MyAz,ffNumber,7,0);

  //  Hops

  if N < 2 then
    Hops := 1
  else
    Hops := Trunc(N);

    //  your_output_display_object := Hops

  Main.edHops.Text := IntToStr(Hops);

end;

procedure StartCalc;
//
//                                              *** PLUG IN YOUR INPUT HERE ***
//
//
//  Get source and destination long and lat, solar flux value, and month,
//  then begin calculations
//
var
  i, j : Integer;

begin

  //  Initialize variables

  Init;

  //  Get destination lat and long

  //  XT := destination latitide, -90.0..+90.0, where positive = N

  XT := StrToFloat(Main.edLatDest.Text);

  //  YT := destination longitude, -180.0..+180.0, where positive = W

  YT := StrToFloat(Main.edLongDest.Text);


  //  Get source lat and long

  //  XR := source latitude

  XR := StrToFloat(Main.edLatSource.Text);

  //  YR := source longitude

  YR := StrToFloat(Main.edLongSource.Text);


  //  Get sunspot number, using F as the current solar flux

  //  R := FluxToSpots(F);

  R := FluxToSpots(StrToFloat(Main.edFlux.Text));

  //  Get month

  //  M := Month;      //  1..12 or 1.0..12.0

  M := StrToInt(Main.edMonth.Text);


  //  Begin calculations

  GeoCalc;
  CalcMyAz;
  CalcOut;

  //  Display the results
  
  ShowResults;

end;

end.


(*

MICROMUF.BAS:


1000 A=100:REM `MICROMUF'
1010 A=0:N=0:YT=0:XT=0:YR=0:XR=0
1020 R=0:M=0:X=0:Y=0:LA=0:LS=0:HP=0
1030 SF=0:FE=0:SE=0:RE=0:CP=0
1040 K=0:HO=0:VE=O:L=0:XZ=0:MF=0:FF=0
1050 XH=0:SX=0:WX=0:LH=0:LM=0:AB=O:T=0
1060 SX=0:WX=0:LH=0:LM=0:AB=0:T=0
1070 T$="":R$="":A$=""
1080 P=3.14159265#
1090 I$="INVALID"
1100 RD=P/180:D=180/P
1110 CLS
1120 PRINT "                    *** MICROMUF ***                       "
1130 PRINT
1140 PRINT "THIS PROGRAM CALCULATES THE:"
1150 PRINT
1160 PRINT " * M. U. F. (MAXIMUM USABLE FREQUENCY)"
1170 PRINT
1180 PRINT " * L. U. F. (LOWEST USABLE  FREQUENCY)"
1190 PRINT
1200 PRINT "OF ANY SHORTWAVE SKYWAVE-PATH."
1210 PRINT :PRINT
1220 PRINT "CALCULATIONS CAN BE DONE FOR"
1230 PRINT
1240 PRINT "ANY MONTH AND SUNSPOT NUMBER."
1250 PRINT :PRINT
1260 PRINT "NAME TRANSMITTER LOCATION"
1270 INPUT T$
1280 PRINT
1290 PRINT "TRANSMITTER LONGITUDE"
1300 PRINT "IN DEGR. (W=+ E=-)";
1310 INPUT YT
1320 IF YT<-180 OR YT>180 THEN PRINT I$:GOTO 1310
1330 PRINT
1340 PRINT "TRANSMITTER LATITUDE"
1350 PRINT "IN DEGR. (N=+ S=-)";
1360 INPUT XT
1370 IF XT<-90 OR XT>90 THEN PRINT I$:GOTO 1360
1380 PRINT
1390 PRINT "NAME RECEIVER LOCATION"
1400 INPUT R$
1410 PRINT
1420 PRINT "RECEIVER LONGITUDE"
1430 PRINT "IN DEGR. (W=+ E=-)";
1440 INPUT YR
1450 IF YR<-180 OR YR>180 THEN PRINT I$:GOTO 1440
1460 PRINT
1470 PRINT "RECEIVER LATITUDE"
1480 PRINT "IN DEGR. (N=+ S=-)";
1490 INPUT XR
1500 IF XR<-90 OR XR>90 THEN PRINT I$:GOTO 1490
1510 PRINT
1520 PRINT "SUNSPOT NUMBER";
1530 INPUT R
1540 IF R<1 OR R>180 THEN PRINT I$:GOTO 1530
1550 PRINT
1560 PRINT "MONTH";
1570 INPUT M
1580 IF M<1 OR M>12 THEN PRINT I$:GOTO 1570

1590 REM *** GEOMETRY CALC
1600 Q=SIN(XT*RD)*SIN(XR*RD)
1610 X=Q+COS(XT*RD)*COS(XR*RD)*COS(YT*RD-YR*RD)
1620 LA=-ATN(X/SQR(-X*X+1+1E-12))+(P/2):LA=LA*D
1630 L=111.1*LA
1640 Q=(SIN(XR*RD)-SIN(XT*RD)*COS(LA*RD))
1650 X=Q/COS(XT*RD)/SIN(LA*RD)
1660 U=-ATN(X/SQR(-X*X+1+1E-12))+(P/2):U=U*D
1670 IF YT-YR<=0 THEN U=360-U
1680 IF ABS(YT-YR)>=180 THEN U=360-U
1690 H=275+R/2
1700 XS=23.4*COS(30*(M-6.25)*RD)
1710 N=N+1
1720 LH=L/N
1730 WHILE LH>4500
1731     N=N+1
1732     LH=L/N
1735 WEND
1740 LM=LA/N
1750 A=ATN((COS(.5*LM*RD)-6367/(H+6367))/SIN(.5*LM*RD))
1760 A=A*D
1770 WHILE A<1.5
1771     N=N+1
1772     LH=L/N
1773     WHILE LH>4500
1774        N=N+1
1775        LH=L/N
1776     WEND
1777     LM=LA/N
1778     A=ATN((COS(.5*LM*RD)-6367/(H+6367))/SIN(.5*LM*RD))
1779     A=A*D
1780 WEND
1781 CLS
1790 PRINT "FROM:";T$;" TO:";R$
1800 PRINT "MONTH:";M;
1810 PRINT " SSN:";R;:PRINT " DIST:";INT(L+.5);"KM"
1820 PRINT "AZIM:";INT(U+.5);"DEGR. F-HOPS:";N
1830 VE=4:HO=1:LOCATE VE,HO
1840 FOR Q=34 TO 2 STEP -2
1850 PRINT "I                         I";Q  '25 SPACES
1860 NEXT Q
1870 PRINT "---------------------------"  '27 DASHES
1880 PRINT " 0 2 4 6 8 10  14  18  22 H(UTC)"
1890 PRINT "      +:MUF  -:LUF"
1895 VE=4:HO=32:LOCATE VE,HO:PRINT "MHZ"
1900 FOR T=1 TO 24
1910 AB=0
1920 K=.5:GOSUB 2120:GOSUB 2240:MF=FF
1930 K=N-.5:GOSUB 2120:GOSUB 2240
1940 IF FF<MF THEN MF=FF
1950 VE=21-INT(MF/2+.5):HO=T+1
1960 IF VE<4 THEN VE=4
1970 LOCATE VE,HO
1980 PRINT "+"
1990 FOR K=.25 TO N-.25 STEP .5
2000 GOSUB 2120
2010 GOSUB 2490
2020 AB=AB+LS
2030 NEXT K
2040 VE=20-INT(AB+.5)
2050 IF VE<4 THEN VE=4
2060 IF VE>20 THEN VE=20
2070 LOCATE VE,HO
2080 PRINT "-"
2090 NEXT T
2100 VE=23:H0=28:LOCATE VE,HO:PRINT "HIT ANY KEY !"
2110 QZ$=INKEY$:IF QZ$="" THEN 2110 ELSE GOTO 1000
2120 REM *** INTERM. LAT. & LONG.
2130 Q=COS(U*RD)*COS(XT*RD)*SIN(K*LM*RD)
2140 X=Q+SIN(XT*RD)*COS(K*LM*RD)
2150 XN=ATN(X/SQR(-X*X+1+1E-12)):XN=XN*D
2160 Q=(COS(K*LM*RD)-SIN(XT*RD)*SIN(XN*RD))
2170 X=Q/(COS(XT*RD)*COS(XN*RD))
2180 YI=-ATN(X/SQR(-X*X+1+1E-12))+(P/2):YI=YI*D
2190 IF U<180 THEN YI=-YI
2200 YN=YT+YI
2210 IF YN>180 THEN YN=YN-360
2220 IF YN<-180 THEN YN=YN+360
2230 RETURN
2240 YZ=YN:REM *** 'MINI-F2' F-LAYER CALC.
2250 IF YN<-160 THEN YZ=YN+360
2260 YG=(20-YZ)/50
2270 ZO=20*YG/(1+YG+YG^2)+5*(1-YG/7)^2
2280 Z=XN-ZO
2290 TL=T-YN/15
2300 IF TL>24 THEN TL=TL-24
2310 IF TL<0 THEN TL=TL+24
2320 MH=M
2330 IF Z>0 THEN 2360
2340 Z=-Z
2350 MH=M+6
2360 XH=COS(30*(MH-6.5)*RD):REM 1 WEEK DELAY ON EQUINOXES
2370 SX=(ABS(XH)+XH)/2:REM F LAYER LOCAL SUMMER VAR.
2380 WX=(ABS(XH)-XH)/2:REM F LAYER LOCAL WINTER VAR.
2390 IF Z>77.5 THEN Z=77.5
2400 TY=TL
2410 IF TY<5 THEN TY=TL+24
2420 YF=(TY-14-SX*2+WX*2-R/175)*(7-SX*3+WX*4-R/(150-WX*75))
2430 IF ABS(YF)>60 THEN YF=60
2440 X=(1+R/(175+SX*175))
2450 F0=6.5*X*COS(YF*RD)*COS((Z-SX*5+WX*5)*RD)^.5
2460 SF=(1-(COS(A*RD)*6367/(6367+H))^2)^-.5
2470 FF=F0*SF:REM MUF AT CONTROL POINT
2480 RETURN
2490 REM *** E-LAYER & ABSORPT. CALC.
2500 Q=SIN(XN*RD)*SIN(XS*RD)
2510 X=Q+COS(XN*RD)*COS(XS*RD)*COS((YN-15*(T-12))*RD)
2520 XZ=(-ATN(X/SQR(-X*X+1+1E-12))+P/2)*D
2530 IF XZ>85 GOTO 2560
2540 FE=3.4*(1+.0016*R)*COS(XZ*RD)^.3
2550 GOTO 2570
2560 FE=3.4*(1+.0016*R)*(XZ-80)^-.5
2570 SE=(1-.965*COS(A*RD)^2)^-.5
2580 LS=.028*FE^2*SE
2590 RETURN


//  Comments from the Turbo Pascal conversion of the program:

{  This program uses 'MINI-F2' devised by R. Fricker (BBC external
  services) for FO-F calculations and L.M. Muggleton's formula for
  FO-E calculations.

   For the L.U.F. a minimum useable fieldstrength of 30 DBUV at the
  receiver and 250 KW of transmitter power (aerial gain: 18 DBI) are
  assumed.  The L.U.F. is derived from absorption calculations based
  on the work of Piggot, George, Samuel, and Bradley.  In spite of the
  program's simplicity it gives a good impression of the ionosphere's
  behaviour and can be used for propagation predictions.

            Hans Bakhuizen
            Propagation Unit; Frequency Bureau
            Radio Netherlands
            P.O. Box 222
            1200 JG Hilversum Holland

(C) Copyright Media Network June 1984



Translation from Basic into TURBO Pascal by Jonathan D Ogden on
September 26, 1986.

       Jonathan D Ogden
       402 e Daniel
       Champaign, Il 61820 USA




Translation to Dephi by Tom Lackamp, ca. 2002.  Originally incorporated
into my Scan320DB program, and tested over several years of shortwave
listening with good results.
 
}


SOLAR FLUX and SUNSPOT NUMBER equivalence:

The following equations are useful for converting between 10 cm flux (F)
and sunspot number (R). The equation are valid on a statistical
(i.e. average) basis.

F = 67.0 + 0.572 R + (0.0575 R)** 2 - (0.0209 R)**3

R = 1.61 FD - (0.0733 FD)** 2 + (0.0240 FD)**3

where, FD = F - 67.0, and ** means "raised to power of".

    (Source:  IPS Radio & Space Services)

     Added to the program by Tom Lackamp, ca 2002

*)


